package com.ruoyi.system.service.impl;

import java.util.*;
import java.util.stream.Collectors;
import javax.validation.Validator;

import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.PinyUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanValidators;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysUserService;

/**
 * 用户 业务层处理
 * 
 * @author ruoyi
 */
@Service
public class SysUserServiceImpl implements ISysUserService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysDeptMapper deptMapper;

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    protected Validator validator;

    @Autowired
    private RecordInfoMapper recordInfoMapper;

    /**
     * 根据条件分页查询用户列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(SysUser user)
    {
        return userMapper.selectUserList(user);
    }

    /**
     * 查询所有人员数量
     * @param kaoQinRecord
     * @return
     */
    @Override
    public Integer selectUserCount(KaoqinRecord kaoQinRecord) {
        return userMapper.selectUserCount(kaoQinRecord);
    }

    /**
     * 根据条件分页查询已分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectAllocatedList(SysUser user)
    {
        return userMapper.selectAllocatedList(user);
    }

    /**
     * 根据条件分页查询未分配用户角色列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUnallocatedList(SysUser user)
    {
        return userMapper.selectUnallocatedList(user);
    }

    /**
     * 登录页重置用户密码
     * @param user 用户信息
     * @return
     */
    @Override
    public int resetPwdLogin(SysUser user) {
        return userMapper.updateUserByUserName(user);
    }

    /**
     * 通过用户名查询用户
     * 
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserByUserName(String userName)
    {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     * 
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId)
    {
        return userMapper.selectUserById(userId);
    }

    /**
     * 查询用户所属角色组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserRoleGroup(String userName)
    {
        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
    }

    /**
     * 查询用户所属岗位组
     * 
     * @param userName 用户名
     * @return 结果
     */
    @Override
    public String selectUserPostGroup(String userName)
    {
        List<SysPost> list = postMapper.selectPostsByUserName(userName);
        if (CollectionUtils.isEmpty(list))
        {
            return StringUtils.EMPTY;
        }
        return list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
    }

    /**
     * 校验用户名称是否唯一
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public String checkUserNameUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkUserNameUnique(user.getUserName());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验手机号码是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkPhoneUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkPhoneUnique(user.getPhonenumber());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验email是否唯一
     *
     * @param user 用户信息
     * @return
     */
    @Override
    public String checkEmailUnique(SysUser user)
    {
        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
        SysUser info = userMapper.checkEmailUnique(user.getEmail());
        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
        {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验用户是否允许操作
     * 
     * @param user 用户信息
     */
    @Override
    public void checkUserAllowed(SysUser user)
    {
        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
        {
            throw new ServiceException("不允许操作超级管理员用户");
        }
    }

    /**
     * 校验用户是否有数据权限
     * 
     * @param userId 用户id
     */
    @Override
    public void checkUserDataScope(Long userId)
    {
        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
        {
            SysUser user = new SysUser();
            user.setUserId(userId);
            List<SysUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
            if (StringUtils.isEmpty(users))
            {
                throw new ServiceException("没有权限访问用户数据！");
            }
        }
    }

    /**
     * 新增保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertUser(SysUser user)
    {
        // 新增用户信息
        int rows = userMapper.insertUser(user);
        // 新增用户岗位关联
        insertUserPost(user);
        // 新增用户与角色管理
        insertUserRole(user);
        return rows;
    }

    /**
     * 新增保存用户信息app
     * @param user 用户信息
     * @return
     */
    @Override
    public int insertUserApp(SysUser user) {
        // 新增用户信息
        return userMapper.insertUser(user);
    }

    /**
     * 注册用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public boolean registerUser(SysUser user)
    {
        return userMapper.insertUser(user) > 0;
    }

    /**
     * 修改保存用户信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateUser(SysUser user)
    {
        Long userId = user.getUserId();
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 新增用户与角色管理
        insertUserRole(user);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPostByUserId(userId);
        // 新增用户与岗位管理
        insertUserPost(user);
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户手机号
     * @param user
     * @return
     */
    @Override
    public int updatePhone(SysUser user) {
        return userMapper.updatePhone(user);
    }

    /**
     * 用户授权角色
     * 
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    @Override
    @Transactional
    public void insertUserAuth(Long userId, Long[] roleIds)
    {
        userRoleMapper.deleteUserRoleByUserId(userId);
        insertUserRole(userId, roleIds);
    }

    /**
     * 修改用户状态
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserStatus(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        return userMapper.updateUser(user);
    }

    /**
     * 修改用户头像
     * 
     * @param userName 用户名
     * @param avatar 头像地址
     * @return 结果
     */
    @Override
    public boolean updateUserAvatar(String userName, String avatar)
    {
        return userMapper.updateUserAvatar(userName, avatar) > 0;
    }

    /**
     * 重置用户密码
     * 
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwd(SysUser user)
    {
        return userMapper.updateUser(user);
    }
    /**
     * 重置用户密码App
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int resetPwdAPP(SysUser user) {
        return userMapper.updateResetPwdAPP(user);
    }

    /**
     * 重置用户密码
     * 
     * @param userName 用户名
     * @param password 密码
     * @return 结果
     */
    @Override
    public int resetUserPwd(String userName, String password)
    {
        return userMapper.resetUserPwd(userName, password);
    }

    /**
     * 新增用户角色信息
     * 
     * @param user 用户对象
     */
    public void insertUserRole(SysUser user)
    {
        this.insertUserRole(user.getUserId(), user.getRoleIds());
    }

    /**
     * 新增用户岗位信息
     * 
     * @param user 用户对象
     */
    public void insertUserPost(SysUser user)
    {
        Long[] posts = user.getPostIds();
        if (StringUtils.isNotEmpty(posts))
        {
            // 新增用户与岗位管理
            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
            for (Long postId : posts)
            {
                SysUserPost up = new SysUserPost();
                up.setUserId(user.getUserId());
                up.setPostId(postId);
                list.add(up);
            }
            userPostMapper.batchUserPost(list);
        }
    }

    /**
     * 新增用户角色信息
     * 
     * @param userId 用户ID
     * @param roleIds 角色组
     */
    public void insertUserRole(Long userId, Long[] roleIds)
    {
        if (StringUtils.isNotEmpty(roleIds))
        {
            // 新增用户与角色管理
            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
            for (Long roleId : roleIds)
            {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(userId);
                ur.setRoleId(roleId);
                list.add(ur);
            }
            userRoleMapper.batchUserRole(list);
        }
    }

    /**
     * 通过用户ID删除用户
     * 
     * @param userId 用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserById(Long userId)
    {
        // 删除用户与角色关联
        userRoleMapper.deleteUserRoleByUserId(userId);
        // 删除用户与岗位表
        userPostMapper.deleteUserPostByUserId(userId);
        return userMapper.deleteUserById(userId);
    }

    /**
     * 批量删除用户信息
     * 
     * @param userIds 需要删除的用户ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteUserByIds(Long[] userIds)
    {
        for (Long userId : userIds)
        {
            checkUserAllowed(new SysUser(userId));
            checkUserDataScope(userId);
        }
        // 删除用户与角色关联
        userRoleMapper.deleteUserRole(userIds);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPost(userIds);
        return userMapper.deleteUserByIds(userIds);
    }

    /**
     * 导入用户数据
     * 
     * @param userList 用户数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName 操作用户
     * @return 结果
     */
    @Override
    public String importUser(List<SysUser> userList, Boolean isUpdateSupport, String operName)
    {
        SysUser use = SecurityUtils.getLoginUser().getUser();

        if (StringUtils.isNull(userList) || userList.size() == 0)
        {
            throw new ServiceException("导入用户数据不能为空！");
        }
        int successNum = 0;
        int failureNum = 0;
        StringBuilder successMsg = new StringBuilder();
        StringBuilder failureMsg = new StringBuilder();
        String password = configService.selectConfigByKey("sys.user.initPassword");
        for (SysUser user : userList)
        {
            try
            {
                // 验证是否存在这个用户
                SysUser u = userMapper.selectUserByUserName(user.getUserName());
                if (StringUtils.isNull(u))
                {
                    BeanValidators.validateWithException(validator, user);
                    user.setDeptId(use.getDeptId());
                    user.setPassword(SecurityUtils.encryptPassword(password));
                    user.setCreateBy(operName);
                    this.insertUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 导入成功");
                }
                else if (isUpdateSupport)
                {
                    BeanValidators.validateWithException(validator, user);
                    checkUserAllowed(user);
                    checkUserDataScope(user.getUserId());
                    user.setDeptId(use.getDeptId());
                    user.setUpdateBy(operName);
                    this.updateUser(user);
                    successNum++;
                    successMsg.append("<br/>" + successNum + "、账号 " + user.getUserName() + " 更新成功");
                }
                else
                {
                    failureNum++;
                    failureMsg.append("<br/>" + failureNum + "、账号 " + user.getUserName() + " 已存在");
                }
            }
            catch (Exception e)
            {
                failureNum++;
                String msg = "<br/>" + failureNum + "、账号 " + user.getUserName() + " 导入失败：";
                failureMsg.append(msg + e.getMessage());
                log.error(msg, e);
            }
        }
        if (failureNum > 0)
        {
            failureMsg.insert(0, "很抱歉，导入失败！共 " + failureNum + " 条数据格式不正确，错误如下：");
            throw new ServiceException(failureMsg.toString());
        }
        else
        {
            successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条，数据如下：");
        }
        return successMsg.toString();
    }


    @Override
    public AjaxResult updateApplyFor(SysUser user, Long id, Long deptId) {
        //查询dept信息
        SysDept dept = deptMapper.selectDeptById(deptId);

        SysUser applyForUser = new SysUser();
        applyForUser.setUserId(id);
        applyForUser.setApplyFor("0");
        userMapper.updateUser(applyForUser);

        //将消息保存进消息记录表
        SysUser users = userMapper.selectUserById(id);
        //SysDept dept = user.getDept();
        RecordInfo recordInfo = new RecordInfo();
        recordInfo.setTitle("部门申请");
        StringBuilder sb = new StringBuilder();
        sb.append(user.getNickName()).append("邀请您加入").append(dept.getDeptName());
        recordInfo.setContent(sb.toString());
        recordInfo.setDeptId(deptId);
        recordInfo.setDeptName(dept.getDeptName());
        //寻找最上级部门
        if("0".equals(dept.getAncestors())){
            recordInfo.setTopDept(dept.getDeptName());
        }else{
            List<String> strList = Arrays.asList(dept.getAncestors().split(","));
            Long topDeptId = Long.valueOf(strList.get(1));
            SysDept sysDeptA = deptMapper.selectDeptById(topDeptId);
            recordInfo.setTopDept(sysDeptA.getDeptName());
        }
        recordInfo.setSenderId(String.valueOf(user.getUserId()));
        recordInfo.setSenderName(user.getNickName());
        recordInfo.setAddresseeId(String.valueOf(id));
        recordInfo.setAddresseeName(users.getNickName());
        recordInfo.setIsRead("0");
        recordInfo.setInfoType("1");
        recordInfo.setCreateBy(user.getUserName());
        recordInfo.setUpdateBy(user.getUserName());
        recordInfoMapper.insertRecordInfo(recordInfo);
        return AjaxResult.success("邀请已发出");
    }

    @Override
    public AjaxResult reply(String status,Long id) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        //查询当前用户最新的一次部门更换邀请
        RecordInfo recordInfo = recordInfoMapper.selectRecordInfoById(id);
        //修改消息状态
        recordInfo.setIsRead(status);
        recordInfoMapper.updateRecordInfo(recordInfo);

        //修改人员应答状态
        SysUser applyForUser = new SysUser();
        if("1".equals(status)){
            applyForUser.setUserId(user.getUserId());
            applyForUser.setDeptId(0L);
            userMapper.updateUser(applyForUser);
            return AjaxResult.success("已拒绝");
        }else{
            applyForUser.setUserId(user.getUserId());
            applyForUser.setDeptId(recordInfo.getDeptId());
            applyForUser.setApplyFor("1");
            userMapper.updateUser(applyForUser);
            //将其他未处理的邀请消息改为已拒绝
            recordInfoMapper.updateIsReadByAddresseeId(String.valueOf(user.getUserId()));

            return AjaxResult.success("已同意");
        }

    }

    @Override
    public AjaxResult query(SysUser user) {
        Map<String,Object> map = new HashMap<>();
        if(user.getDeptId()==0){
            map.put("info","");
            map.put("status","0");
            return AjaxResult.success(map);
        }
        //获取变更部门信息
        SysDept dept = deptMapper.selectDeptById(user.getDeptId());
        StringBuilder sb = new StringBuilder();
        sb.append(dept.getDeptName()).append("邀请您加入该部门");
        map.put("info",sb.toString());
        map.put("status","1");
        return AjaxResult.success(map);
    }

    @Override
    public AjaxResult allocationOne(String phonenumber) {
        if(StringUtils.isEmpty(phonenumber)){
            return AjaxResult.error("手机号不能为空");
        }
        SysUser applyForUser = userMapper.selectUserByPhonenumber(phonenumber);
        if (applyForUser!=null){
            return AjaxResult.success("操作成功",applyForUser);
        }
        return AjaxResult.success("该用户尚未注册或已同意邀请");
    }

    @Override
    public int reset(Long id) {

        // 重置用户与角色关联（先删后新增）
        userRoleMapper.deleteUserRoleByUserId(id);
        List<SysUserRole> list = new ArrayList<SysUserRole>();
        SysUserRole ur = new SysUserRole();
        ur.setUserId(id);
        ur.setRoleId(2L);
        list.add(ur);
        userRoleMapper.batchUserRole(list);
        // 删除用户与岗位关联
        userPostMapper.deleteUserPostByUserId(id);
        //将deptId置0
        //修改人员应答状态
        SysUser applyForUser = new SysUser();
        applyForUser.setUserId(id);
        applyForUser.setDeptId(0L);
        applyForUser.setApplyFor("0");
        return userMapper.updateUser(applyForUser);
    }

    @Override
    public List<SysUser> selectUserListNoZj(SysUser user) {
        return userMapper.selectUserListNoZj(user);
    }

    @Override
    public AjaxResult szmList(SysUser user) {
        Map<String,Object> map = new HashMap<>();
        List<SysUser> list = userMapper.selectUserList(user);
        map.put("num",list.size());
        List<Map<String,Object>> resultList = new ArrayList<>();

        // 使用TreeMap自动按键排序（字母顺序）
        Map<Character, List<SysUser>> classifiedMap = new TreeMap<>();
        for (SysUser person : list) {
            char initial = PinyUtils.getPinyinInitial(person.getNickName());

            // 跳过无效姓名
            if (initial == ' ') continue;

            // 初始化或获取对应首字母的列表
            List<SysUser> group = classifiedMap.computeIfAbsent(initial, k -> new ArrayList<>());
            group.add(person);
        }
        for (Map.Entry<Character, List<SysUser>> entry : classifiedMap.entrySet()) {
            Map<String, Object> groupMap = new LinkedHashMap<>();
            groupMap.put("initial", String.valueOf(entry.getKey()));  // 首字母作为字符串
            groupMap.put("list", entry.getValue());  // 人员列表

            resultList.add(groupMap);
        }
        map.put("list",resultList);
        return AjaxResult.success(map);
    }



}
